//---------------------------------------------------------------------------
// TestStorm.cpp
//---------------------------------------------------------------------------
#include "TestStorm.h"
#include "SimManager.h"
#include "GridBase.h"
#include "Storm.h"
#include <fstream>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
//  TestStorm()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestStorm()
{
  clSimManager * p_oSimManager = new clSimManager( 3, 2, "" );

  TestNormalDeterministic( p_oSimManager );
  TestNormalMappedDeterministic( p_oSimManager );
  TestNormalStochastic( p_oSimManager );
  TestNormalMappedStochastic( p_oSimManager );
  TestErrorProcessing( p_oSimManager );
  TestMultiYearTimesteps( p_oSimManager );

  delete p_oSimManager;
}

/////////////////////////////////////////////////////////////////////////////
//  TestNormalDeterministic()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestNormalDeterministic( clSimManager * p_oSimManager )
{
  clGridBase * p_oStormDamage; //storm damage grid
  clBehaviorBase * p_oTemp;
  clStorm * p_oStormBehavior;
  float fVal;
  int i, iX, iY, iNumXCells, iNumYCells, iIndexCode, iStormTimeCode;

  //Load our parameter file
  p_oSimManager->ReadFile( WriteXMLFile1() );

  //Get the grid object and make sure it's set up correctly
  p_oStormDamage = p_oSimManager->GetGridObject( "Storm Damage" );
  CPPUNIT_ASSERT( p_oStormDamage != NULL );
  iNumXCells = p_oStormDamage->GetNumberXCells();
  iNumYCells = p_oStormDamage->GetNumberYCells();
  CPPUNIT_ASSERT( 13 == iNumXCells );
  CPPUNIT_ASSERT( 17 == iNumYCells );
  iIndexCode = p_oStormDamage->GetFloatDataCode( "dmg_index" );
  iStormTimeCode = p_oStormDamage->GetFloatDataCode( "stormtime" );
  CPPUNIT_ASSERT( iIndexCode != -1 );
  CPPUNIT_ASSERT( iStormTimeCode != -1 );

  //Make sure the storm probabilities were calculated correctly
  p_oTemp = p_oSimManager->GetBehaviorObject( "storm" );
  CPPUNIT_ASSERT( NULL != p_oTemp );
  p_oStormBehavior = dynamic_cast<clStorm *>(p_oTemp);
  CPPUNIT_ASSERT( fabs(0.5 - p_oStormBehavior->mp_fStormProbabilities[0] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(1 - p_oStormBehavior->mp_fStormProbabilities[1] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.05 - p_oStormBehavior->mp_fStormProbabilities[2] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0 - p_oStormBehavior->mp_fStormProbabilities[3] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.075757576 - p_oStormBehavior->mp_fStormProbabilities[4] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.022222222 - p_oStormBehavior->mp_fStormProbabilities[5] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.02 - p_oStormBehavior->mp_fStormProbabilities[6] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.018181818 - p_oStormBehavior->mp_fStormProbabilities[7] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.016666667 - p_oStormBehavior->mp_fStormProbabilities[8] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.015384615 - p_oStormBehavior->mp_fStormProbabilities[9] ) < 0.00001);

  // ****************************
  // Timestep 1
  // ****************************
  //Make sure no storms will occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 1 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 1 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 2
  // ****************************
  //Rig so a storm of severity 0.4-0.5 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[4] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 2 results - all cells should be between 0.4 and 0.5
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal >= 0.4 && fVal <= 0.5);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 3
  // ****************************
  //Rig so 2 storms of severity 0.2-0.3 and 0.5-0.6 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[2] = 1;
  p_oStormBehavior->mp_fStormProbabilities[5] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 3 results - all cells should be between 0.7 and 0.9
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal >= 0.7 && fVal <= 0.9);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 4
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  p_oSimManager->RunSim( 1 );

  //Check timestep 4 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 1 == fVal );
    }

  // ****************************
  // Timestep 5
  // ****************************
  //Rig so 2 storms of of severity 0.6-0.7 and 0.8-0.9 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[6] = 1;
  p_oStormBehavior->mp_fStormProbabilities[8] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 5 results - all cells should be 1
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(1 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
  
  // ****************************
  // Timestep 6
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  p_oSimManager->RunSim( 1 );

  //Check timestep 6 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 1 == fVal );
    }
    
  // ****************************
  // Timestep 7
  // ****************************
    p_oSimManager->RunSim( 1 );

  //Check timestep 7 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 2 == fVal );
    }
  
  // ****************************
  // Timestep 8
  // ****************************
  p_oSimManager->RunSim( 1 );

  //Check timestep 8 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 3 == fVal );
    }
}

/////////////////////////////////////////////////////////////////////////////
//  TestNormalMappedDeterministic()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestNormalMappedDeterministic( clSimManager * p_oSimManager )
{
  clGridBase * p_oStormDamage, //storm damage grid
       * p_oSusceptibility;
  clBehaviorBase * p_oTemp;
  clStorm * p_oStormBehavior;
  float fVal, fExpectedValMin, fExpectedValMax;
  int i, iX, iY, iNumXCells, iNumYCells, iIndexCode;

  //Load our parameter file
  p_oSimManager->ReadFile( WriteXMLFile2() );

  //Get the grid object and make sure it's set up correctly
  p_oStormDamage = p_oSimManager->GetGridObject( "Storm Damage" );
  CPPUNIT_ASSERT( p_oStormDamage != NULL );
  iNumXCells = p_oStormDamage->GetNumberXCells();
  iNumYCells = p_oStormDamage->GetNumberYCells();
  CPPUNIT_ASSERT( 20 == iNumXCells );
  CPPUNIT_ASSERT( 44 == iNumYCells );
  iIndexCode = p_oStormDamage->GetFloatDataCode( "dmg_index" );
  CPPUNIT_ASSERT( iIndexCode != -1 );

  p_oSusceptibility = p_oSimManager->GetGridObject( "Storm Susceptibility" );
  CPPUNIT_ASSERT( p_oSusceptibility != NULL );
  iNumXCells = p_oSusceptibility->GetNumberXCells();
  iNumYCells = p_oSusceptibility->GetNumberYCells();
  CPPUNIT_ASSERT( 20 == iNumXCells );
  CPPUNIT_ASSERT( 44 == iNumYCells );

  //Make sure the storm probabilities were calculated correctly
  p_oTemp = p_oSimManager->GetBehaviorObject( "storm" );
  CPPUNIT_ASSERT( NULL != p_oTemp );
  p_oStormBehavior = dynamic_cast<clStorm *>(p_oTemp);

  // ****************************
  // Timestep 1
  // ****************************
  //Rig so a storm of severity 0.1-0.2 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[1] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 1 results - all cells should be between (x/10 + y/10) * 0.1
  //and (x/10 + y/100) * 0.2
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      fExpectedValMin = ( ( float )iX / 10 + ( float )iY / 10 );
      if (fExpectedValMin > 1) fExpectedValMin = 1;
      fExpectedValMin *= 0.1;
      fExpectedValMax =  ( ( float )iX / 10 + ( float )iY / 10 );
      if (fExpectedValMax > 1) fExpectedValMax = 1;
      fExpectedValMax *= 0.2;
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      if (fVal < fExpectedValMin || fVal > fExpectedValMax ) {
        fVal = fVal;
      }
      CPPUNIT_ASSERT( fVal >= fExpectedValMin && fVal <= fExpectedValMax );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 2
  // ****************************
  //Rig so 3 storms of 0-0.1, 0.1-0.2, and 0.2-0.3 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[0] = 1;
  p_oStormBehavior->mp_fStormProbabilities[1] = 1;
  p_oStormBehavior->mp_fStormProbabilities[2] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 2 results - all cells should be between (x/10 + y/10) * 0.3
  //and (x/10 + y/100) * 0.6
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      fExpectedValMin = ( ( float )iX / 10 + ( float )iY / 10 );
      if (fExpectedValMin > 1) fExpectedValMin = 1;
      fExpectedValMin *= 0.3;
      fExpectedValMax =  ( ( float )iX / 10 + ( float )iY / 10 );
      if (fExpectedValMax > 1) fExpectedValMax = 1;
      fExpectedValMax *= 0.6;
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal >= fExpectedValMin && fVal <= fExpectedValMax );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 3
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 3 results - all cells should be 0.
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 4
  // ****************************
  //Rig so 2 storms of severity 0.6-0.7 and 0.9 - 1 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[6] = 1;
  p_oStormBehavior->mp_fStormProbabilities[9] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 4 results - all cells should be between x/10 + y/10 * 1.5
  //and 1
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      fExpectedValMin = ( ( float )iX / 10 + ( float )iY / 10 ) * 1.5;
      if (fExpectedValMin > 1) fExpectedValMin = 1;
      fExpectedValMax = 1;
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal >= fExpectedValMin && fVal <= fExpectedValMax );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
  
  // ****************************
  // Timestep 5
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 5 results - all cells should be 0.
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
  
  // ****************************
  // Timestep 6
  // ****************************
  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 6 results - all cells should be 0.
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
}

/////////////////////////////////////////////////////////////////////////////
//  TestNormalStochastic()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestNormalStochastic( clSimManager * p_oSimManager )
{
  clGridBase * p_oStormDamage; //storm damage grid
  clBehaviorBase * p_oTemp;
  clStorm * p_oStormBehavior;
  float fVal;
  int i, iX, iY, iNumXCells, iNumYCells, iIndexCode;

  //Load our parameter file
  p_oSimManager->ReadFile( WriteXMLFile3() );

  p_oTemp = p_oSimManager->GetBehaviorObject( "storm" );
  CPPUNIT_ASSERT( NULL != p_oTemp );
  p_oStormBehavior = dynamic_cast<clStorm *>(p_oTemp);

  //Get the grid object and make sure it's set up correctly
  p_oStormDamage = p_oSimManager->GetGridObject( "Storm Damage" );
  CPPUNIT_ASSERT( p_oStormDamage != NULL );
  iNumXCells = p_oStormDamage->GetNumberXCells();
  iNumYCells = p_oStormDamage->GetNumberYCells();
  CPPUNIT_ASSERT( 10 == iNumXCells );
  CPPUNIT_ASSERT( 13 == iNumYCells );
  iIndexCode = p_oStormDamage->GetFloatDataCode( "dmg_index" );
  CPPUNIT_ASSERT( iIndexCode != -1 );

  // ****************************
  // Timestep 1
  // ****************************
  //Make sure no storms will occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 1 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 2
  // ****************************
  //Rig so 2 storms of severity 0.6-0.7 and 0.8-0.9 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[6] = 1;
  p_oStormBehavior->mp_fStormProbabilities[8] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 2 results - all cells should be 1 or less
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( 1 >= fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 3
  // ****************************
  //Rig so 1 storm of severity 0.4 - 0.5 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[4] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 3 results - all cells should be a mean of between 0.4 and 0.5
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( 0.1 <= fVal && 0.8 >= fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
}

/////////////////////////////////////////////////////////////////////////////
//  TestNormalMappedStochastic()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestNormalMappedStochastic( clSimManager * p_oSimManager )
{
  clGridBase * p_oStormDamage, //storm damage grid
       * p_oSusceptibility;
  clBehaviorBase * p_oTemp;
  clStorm * p_oStormBehavior;
  float fVal;
  int i, iX, iY, iNumXCells, iNumYCells, iIndexCode;

  //Load our parameter file
  p_oSimManager->ReadFile( WriteXMLFile4() );

  //Get the grid object and make sure it's set up correctly
  p_oStormDamage = p_oSimManager->GetGridObject( "Storm Damage" );
  CPPUNIT_ASSERT( p_oStormDamage != NULL );
  iNumXCells = p_oStormDamage->GetNumberXCells();
  iNumYCells = p_oStormDamage->GetNumberYCells();
  CPPUNIT_ASSERT( 20 == iNumXCells );
  CPPUNIT_ASSERT( 44 == iNumYCells );
  iIndexCode = p_oStormDamage->GetFloatDataCode( "dmg_index" );
  CPPUNIT_ASSERT( iIndexCode != -1 );

  p_oSusceptibility = p_oSimManager->GetGridObject( "Storm Susceptibility" );
  CPPUNIT_ASSERT( p_oSusceptibility != NULL );
  iNumXCells = p_oSusceptibility->GetNumberXCells();
  iNumYCells = p_oSusceptibility->GetNumberYCells();
  CPPUNIT_ASSERT( 20 == iNumXCells );
  CPPUNIT_ASSERT( 44 == iNumYCells );

  //Make sure the storm probabilities were calculated correctly
  p_oTemp = p_oSimManager->GetBehaviorObject( "storm" );
  CPPUNIT_ASSERT( NULL != p_oTemp );
  p_oStormBehavior = dynamic_cast<clStorm *>(p_oTemp);

  // ****************************
  // Timestep 1
  // ****************************
  //Rig so 2 storms of severity 0.7-0.8 and 0.8-0.9 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[7] = 1;
  p_oStormBehavior->mp_fStormProbabilities[8] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 2 results - all cells should have means are between
  //(x/10+y/10)*1.5 and 1.
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal <= 1 && fVal >= 0 );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 2
  // ****************************
  //Rig so 1 storm of severity 0.3-0.4 occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[3] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 2 results - all cell means between (x/10+y/10)*0.3
  //and (x/10+y/10)*0.4
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal <= 1 && fVal >= 0);
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }

  // ****************************
  // Timestep 3
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 3 results - all cells should be 0.
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
}

/////////////////////////////////////////////////////////////////////////////
//  TestMultiYearTimesteps()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestMultiYearTimesteps( clSimManager * p_oSimManager )
{
  clGridBase * p_oStormDamage; //storm damage grid
  clBehaviorBase * p_oTemp;
  clStorm * p_oStormBehavior;
  float fVal;
  int i, iX, iY, iNumXCells, iNumYCells, iIndexCode, iStormTimeCode;

  //Load our parameter file
  p_oSimManager->ReadFile( WriteXMLFile5() );

  //Get the grid object and make sure it's set up correctly
  p_oStormDamage = p_oSimManager->GetGridObject( "Storm Damage" );
  CPPUNIT_ASSERT( p_oStormDamage != NULL );
  iNumXCells = p_oStormDamage->GetNumberXCells();
  iNumYCells = p_oStormDamage->GetNumberYCells();
  CPPUNIT_ASSERT( 13 == iNumXCells );
  CPPUNIT_ASSERT( 17 == iNumYCells );
  iIndexCode = p_oStormDamage->GetFloatDataCode( "dmg_index" );
  iStormTimeCode = p_oStormDamage->GetFloatDataCode( "stormtime");
  CPPUNIT_ASSERT( iIndexCode != -1 );
  CPPUNIT_ASSERT( iStormTimeCode != -1);

  //Make sure the storm probabilities were calculated correctly
  p_oTemp = p_oSimManager->GetBehaviorObject( "storm" );
  CPPUNIT_ASSERT( NULL != p_oTemp );
  p_oStormBehavior = dynamic_cast<clStorm *>(p_oTemp);
  CPPUNIT_ASSERT( fabs(0.5 - p_oStormBehavior->mp_fStormProbabilities[0] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(1 - p_oStormBehavior->mp_fStormProbabilities[1] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.05 - p_oStormBehavior->mp_fStormProbabilities[2] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.033333333 - p_oStormBehavior->mp_fStormProbabilities[3] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.075757576 - p_oStormBehavior->mp_fStormProbabilities[4] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.022222222 - p_oStormBehavior->mp_fStormProbabilities[5] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.02 - p_oStormBehavior->mp_fStormProbabilities[6] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.018181818 - p_oStormBehavior->mp_fStormProbabilities[7] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.016666667 - p_oStormBehavior->mp_fStormProbabilities[8] ) < 0.00001);
  CPPUNIT_ASSERT( fabs(0.015384615 - p_oStormBehavior->mp_fStormProbabilities[9] ) < 0.00001);

  // ****************************
  // Timestep 1
  // ****************************
  //Rig so 2 storms of severity 0.2-0.3. occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[2] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 1 results - all cells should be between 0.4 and 0.6
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal >= 0.4 && fVal <= 0.6);
      
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
  
  // ****************************
  // Timestep 2
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  p_oSimManager->RunSim( 1 );

  //Check timestep 2 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 2 == fVal );
    }
    
  // ****************************
  // Timestep 3
  // ****************************
  //Rig so 2 storms of severity 0.2-0.3. occurs
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }
  p_oStormBehavior->mp_fStormProbabilities[2] = 1;

  //Don't let cleanup occur yet!
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->Action();
  }

  //Check timestep 3 results - all cells should be between 0.4 and 0.6
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fVal >= 0.4 && fVal <= 0.6);
      
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 0 == fVal );
    }

  //Now cleanup
  for ( i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++ )
  {
    p_oSimManager->GetBehaviorObject( i )->TimestepCleanup();
  }
  
  // ****************************
  // Timestep 4
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  p_oSimManager->RunSim( 1 );

  //Check timestep 4 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 2 == fVal );
    }
    
  // ****************************
  // Timestep 5
  // ****************************
  //Rig so no storms occur
  for ( i = 0; i < 10; i++ )
  {
    p_oStormBehavior->mp_fStormProbabilities[i] = 0;
  }

  p_oSimManager->RunSim( 1 );

  //Check timestep 5 results - all cells should be 0
  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      p_oStormDamage->GetValueOfCell( iX, iY, iIndexCode, & fVal );
      CPPUNIT_ASSERT( fabs(0 - fVal ) < 0.00001);
      p_oStormDamage->GetValueOfCell( iX, iY, iStormTimeCode, & fVal );
      CPPUNIT_ASSERT( 4 == fVal );
    }
}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestStorm::TestErrorProcessing(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clStorm::DoGridSetup") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile2());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile2.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clStorm::DoGridSetup") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile2.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile3());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile3.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clStorm::ReadParFile") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile3.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile4());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile4.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clStorm::ReadParFile") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile4.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile5());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile5.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "FillSingleValue") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile5.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile6());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile6.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clStorm::ReadParFile") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile6.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile7());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile7.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "FillSingleValue") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile7.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile8());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile8.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "FillSingleValue") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile8.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile9());
    CppUnit::Asserter::fail("TestStorm error processing didn't catch error for WriteXMLErrorFile9.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clStorm::ReadParFile") == NULL)
    {
      CppUnit::Asserter::fail("TestStorm error processing caught wrong error for WriteXMLErrorFile9.");
    }
    ;
  }
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>5</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<storm>"
       << "<st_s1ReturnInterval>2</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>1</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>20</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>1.00E+28</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>13.2</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>45</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>50</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>55</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>60</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>65</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}



/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>4</timesteps>" 
       << "<yearsPerTimestep>1</yearsPerTimestep>" 
       << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  //Generate the storm susceptibility map
  oOut << "<grid gridName=\"Storm Susceptibility\">" 
       << "<ma_floatCodes>" 
       << "<ma_floatCode label=\"index\">0</ma_floatCode>"
       << "</ma_floatCodes>" 
       << "<ma_lengthXCells>5</ma_lengthXCells>" 
       << "<ma_lengthYCells>3</ma_lengthYCells>";

  //Auto-generate grid values
  int iNumX = 20, iNumY = 44, iX, iY;
  float fValue;
  for ( iX = 0; iX < iNumX; iX++ )
  {
    for ( iY = 0; iY < iNumY; iY++ )
    {
      fValue = ( ( float )iX / 10 ) + ( ( float )iY / 10 );
      if (fValue > 1) fValue = 1;
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">" << "<fl c=\"0\">" << fValue << "</fl>" << "</ma_v>";
    }
  }
  oOut << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>0</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>3</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  //Generate the storm susceptibility map
  oOut << "<grid gridName=\"Storm Damage\">" 
       << "<ma_floatCodes>" 
       << "<ma_floatCode label=\"dmg_index\">0</ma_floatCode>"
       << "<ma_floatCode label=\"stormtime\">0</ma_floatCode>"
       << "</ma_floatCodes>" 
       << "<ma_lengthXCells>10</ma_lengthXCells>" 
       << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>1</st_stochasticity>"
       << "<st_probFunction>1</st_probFunction>"
       << "<st_standardDeviation>0.1</st_standardDeviation>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>4</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  //Generate the storm susceptibility map
  oOut << "<grid gridName=\"Storm Susceptibility\">" << "<ma_floatCodes>" << "<ma_floatCode label=\"index\">0</ma_floatCode>"
       << "</ma_floatCodes>" << "<ma_lengthXCells>5</ma_lengthXCells>" << "<ma_lengthYCells>3</ma_lengthYCells>";

  //Auto-generate grid values
  int iNumX = 20, iNumY = 44, iX, iY;
  float fValue;
  for ( iX = 0; iX < iNumX; iX++ )
  {
    for ( iY = 0; iY < iNumY; iY++ )
    {
      fValue = ( ( float )iX / 10 ) + ( ( float )iY / 10 );
      if (fValue > 1) fValue = 1;
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">" << "<fl c=\"0\">" << fValue << "</fl>" << "</ma_v>";
    }
  }
  oOut << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>0</st_susceptibility>"
       << "<st_stochasticity>1</st_stochasticity>"
       << "<st_probFunction>1</st_probFunction>"
       << "<st_standardDeviation>0.1</st_standardDeviation>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" 
       << "<paramFile fileCode=\"06010101\">" 
       << "<plot>"
       << "<timesteps>9</timesteps>" 
       << "<yearsPerTimestep>2</yearsPerTimestep>" 
       << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<storm>"
       << "<st_s1ReturnInterval>2</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>1</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>20</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>30</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>13.2</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>45</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>50</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>55</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>60</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>65</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestStorm::WriteCommonStuff( std::fstream &oOut )
{
  using namespace std;
  oOut << "<plot_lenX>100</plot_lenX>" 
       << "<plot_lenY>130</plot_lenY>" 
       << "<plot_latitude>55.37</plot_latitude>" 
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>storm</behaviorName>"
       << "<version>1</version>"
       << "</behavior>"
       << "</behaviorList>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>4</timesteps>" << "<yearsPerTimestep>7</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  //Generate the storm susceptibility map
  oOut << "<grid gridName=\"Storm Susceptibility\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"index\">0</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>5</ma_lengthXCells>"
       << "<ma_lengthYCells>3</ma_lengthYCells>";

  //Auto-generate grid values
  int iNumX = 20, iNumY = 44, iX, iY;
  float fValue;
  for ( iX = 0; iX < iNumX; iX++ )
  {
    for ( iY = 0; iY < iNumY; iY++ )
    {
      fValue = 0.5;
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">" << "<fl c=\"0\">" << fValue << "</fl>" << "</ma_v>";
    }
  }
  oOut << "</grid>"
       << "<grid gridName=\"Storm Damage\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"dmg_index\">0</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>10</ma_lengthXCells>"
       << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>"
       << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>0</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>5</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>" << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>" << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>" << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>" << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>" << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>0</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>5</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>" << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>" << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>-0.5</st_s5ReturnInterval>" << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>" << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>" << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>5</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>" << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>" << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>" << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>" << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>" << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>3</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}


/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>5</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>" << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>" << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>" << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>" << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>" << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>1</st_stochasticity>"
       << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile6()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>3</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<grid gridName=\"Storm Damage\">" << "<ma_floatCodes>" << "<ma_floatCode label=\"dmg_index\">0</ma_floatCode>"
       << "</ma_floatCodes>" << "<ma_lengthXCells>10</ma_lengthXCells>" << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>1</st_stochasticity>"
       << "<st_probFunction>9</st_probFunction>" << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile7()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile7()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>3</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<grid gridName=\"Storm Damage\">" << "<ma_floatCodes>" << "<ma_floatCode label=\"dmg_index\">0</ma_floatCode>"
       << "</ma_floatCodes>" << "<ma_lengthXCells>10</ma_lengthXCells>" << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>1</st_stochasticity>"
       << "<st_probFunction>1</st_probFunction>" << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile8()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile8()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>3</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<grid gridName=\"Storm Damage\">" << "<ma_floatCodes>" << "<ma_floatCode label=\"dmg_index\">0</ma_floatCode>"
       << "</ma_floatCodes>" << "<ma_lengthXCells>10</ma_lengthXCells>" << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>1</st_stochasticity>"
       << "<st_probFunction>2</st_probFunction>" << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile9()
/////////////////////////////////////////////////////////////////////////////
char * clTestStorm::WriteXMLErrorFile9()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>3</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>";

  WriteCommonStuff( oOut );

  oOut << "<grid gridName=\"Storm Damage\">" << "<ma_floatCodes>" << "<ma_floatCode label=\"dmg_index\">0</ma_floatCode>"
       << "</ma_floatCodes>" << "<ma_lengthXCells>10</ma_lengthXCells>" << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>" << "<storm>" << "<st_s1ReturnInterval>0.1</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>0.2</st_s2ReturnInterval>" << "<st_s3ReturnInterval>0.3</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>0.4</st_s4ReturnInterval>" << "<st_s5ReturnInterval>0.5</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>0.6</st_s6ReturnInterval>" << "<st_s7ReturnInterval>0.7</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>0.8</st_s8ReturnInterval>" << "<st_s9ReturnInterval>0.9</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>3</st_stochasticity>"
       << "<st_probFunction>2</st_probFunction>" << "</storm>" << "</paramFile>";

  oOut.close();

  return cFileString;
}


